@@ -5,15 +5,11 @@ Win32::Process::Info - Provide process information for Windows 32 systems.
 =head1 SYNOPSIS
 
  use Win32::Process::Info;
- $pi = Win32::Process::Info->new ();
- $pi->Set (elapsed_in_seconds => 0);	# In clunks, not seconds.
+ $pi = Win32::Process::Info->new ([machine], [variant]);
+ $pi->Set (elapsed_as_seconds => 0);	# In clunks, not seconds.
  @pids = $pi->ListPids ();	# Get all known PIDs
  @info = $pi->GetProcInfo ();	# Get the max
  %subs = $pi->Subprocesses ();	# Figure out subprocess relationships.
- @info = grep {
-     defined $_->{Name} &&
-     $_->{Name} =~ m/perl/
- } $pi->GetProcInfo ();        # All processes with 'perl' in name.
 
 =head1 NOTICE
 
@@ -36,12 +32,9 @@ documentation.
 
 This change is somewhat incompatible with 1.006 and earlier because it
 requires the import() method to be called in the correct place with the
-correct arguments. If you C<require Win32::Process::Info>, you B<must>
-explicitly call Win32::Process::Info->import().
+correct arguments. See the import() documentation below for the details.
 
-See the import() documentation below for the details.
-
-B<YOU HAVE BEEN WARNED!>
+B<YOU HAVE BEEN WARNED!>.
 
 =head1 DESCRIPTION
 
@@ -50,10 +43,6 @@ information is convenient (for the author!) about one or more Windows
 32 processes. L</GetProcInfo> is therefore the most-important
 method in the package. See it for more information.
 
-The process IDs made available are those returned by the variant in
-use. See the documentation to the individual variants for details,
-especially if you are a Cygwin user.
-
 Unless explicitly stated otherwise, modules, variables, and so
 on are considered private. That is, the author reserves the right
 to make arbitrary changes in the way they work, without telling
@@ -67,20 +56,139 @@ The following methods should be considered public:
 
 =cut
 
+#	Modifications:
+
+# 0.010	02-Sep-2002	T. R. Wyant
+#		Renamed from Win32::ProcInfo
+#		Initial release under name Win32::Process::Info
+#
+# 0.011	14-Sep-2002	T. R. Wyant
+#		Incremented version number
+#		Added method "Version".
+#
+# 0.012	06-Nov-2002	T. R. Wyant
+#		Incremented version number
+#		Made attributes beginning with "_" hidden.
+#		Add attribute _mutator, containing a reference to
+#			%mutator.
+#
+#	12-Nov-2002	T. R. Wyant
+#		Build hash argument for subclass instantiators.
+#		Document this, and the use of the various keys.
+#
+#	07-Dec-2002	T. R. Wyant
+#		Check environment variable
+#		PERL_WIN32_PROCESS_INFO_VARIANT for the default
+#		variant(s), if none is specified.
+#
+# 0.013	13-Mar-2003	T. R. Wyant
+#		Remove dependencies on Win32::API, Win32::OLE, and
+#			Win32API::Registry from Makefile.PL, since
+#			these are conditional.
+#
+# 0.013_2 21-May-2003	T. R. Wyant
+#		Add the %variant_support hash in response to INIT block
+#			errors in Win32::API when (effectively)
+#			require-d when the object is new-ed.
+#
+# 0.013_21 05-Jun-2003	T. R. Wyant
+#		Found a need to tweak the WMI variant. Wouldn't have
+#			incremented here, but wanted a different package
+#			number since I had previously send out 0.013_2
+#			privately. The tweak was the manual skipping of
+#			processes in response to the contents of
+#			$ENV{PERL_WIN32_PROCESS_INFO_WMI_PARIAH}
+#
+# 0.013_22 20-Jun-2003	T. R. Wyant
+#		No longer assert the debug privilege by default. Only
+#			do it $ENV{PERL_WIN32_PROCESS_INFO_WMI_DEBUG}
+#			is TRUE in the Perl sense.
+#		As a consequence of the effects of the above, default
+#			the pariah list to empty.
+#		Enhanced the test to skip the NT variant if it can't
+#			find all the DLLs.
+#
+# 0.013_23 26-Jun-2003	T. R. Wyant
+#		Added method Subprocesses. Because I needed it.
+#
+# 0.014	27-Jun-2003	T. R. Wyant
+#		Wrapped up latest version, released to CPAN.
+#
+# 0.014_01 25-Jul-2003	T. R. Wyant
+#		Straightened out some of the POD.
+#		Added hash argument assert_debug_priv to 'new' method.
+#			This is legal for all variants, but only
+#			affects the WMI variant.
+# 0.014_02 16-Aug-2003	T. R. Wyant
+#		Added missing semicolon after "use Win32::Process::Info"
+#			in the synopsis.
+#
+# 1.000 09-Oct-2003	T. R. Wyant
+#		When the only thing you've fixed in the last two months
+#			is the docs, it's time to call it production
+#			code. And if _that_ statement doesn't flush
+#			out more problems, nothing will.
+#
+# 1.001	05-Jan-2004	T. R. Wyant
+#		No changes to the code itself. But removed the
+#			dependency on Win32 in Makefile.PL because
+#			PPM3 chokes on it, and I figure anyone who isn't
+#			using PPM is smart enough to read the Readme.
+#
+# 1.001_01 24-Feb-2004	T. R. Wyant
+#		Failed attempt to fix WMI memory leak. But it's clearly
+#		associated with the use of Win32::OLE::Variant objects
+#		to retrieve the owner information, because the code
+#		appears not to leak if we don't do this. Versions prior
+#		to this one leaked anyway, because the Variant objects
+#		were allocated whether or not they were used. But this
+#		version doesn't allocate them unless they are to be
+#		used.
+#
+# 1.002	07-Jun-2004	T. R. Wyant
+#		Document leaks under WMI, and how to minimize. Document
+#		related modules.
+#
+# 1.003 19-Dec-2004	T. R. Wyant
+#		Remove the "_PRIV" from the end of environment variable
+#		name PERL_WIN32_PROCESS_INFO_WMI_DEBUG in the POD.
+#		Record the variant name in the object.
+#		Clarify (hopefully) the PERL_WIN32_PROCESS_INFO_VARIANT
+#		documentation.
+#		Document current whereabouts of Win32::IProc - or at
+#		least the sad remnants of it.
+# 1.004	30-Dec-2004	T. R. Wyant
+#		No code change. Removed the commented-out dependencies
+#		in Makefile.PL, since sometime in the last 6 months
+#		someone has enhanced CPAN's validation to generate
+#		a bug report when I do this. The only hard-and-fast
+#		dependency is on Win32, but ActiveState's PPM3
+#		chokes on this.
+#
+# 1.005 15-Mar-2005	T. R. Wyant
+#		Moved assertion of seDebugPriv in the NT variant to
+#		stop token handle leak.
+#		Turned off $^W for timelocal call, since it throws
+#		random warnings otherwise.
+# 1.006 23-Sep-2005 T. R. Wyant
+#		Skip non-existent processes in the Subprocesses method.
+# 1.007 10-Jan-2007	T. R. Wyant
+#		Use the import() hook to determine which variants may
+#		be used in the script.
+
 package Win32::Process::Info;
 
-use 5.006;
+$VERSION = '1.007';
 
 use strict;
-use warnings;
-
-our $VERSION = '1.020_01';
+use vars qw{%mutator %static};
 
 use Carp;
 use File::Spec;
 use Time::Local;
+use UNIVERSAL qw{isa};
 
-our %static = (
+%static = (
     elapsed_in_seconds	=> 1,
     variant		=> $ENV{PERL_WIN32_PROCESS_INFO_VARIANT},
     );
@@ -105,109 +213,65 @@ our %static = (
 
 my %variant_support;
 BEGIN {
-    # Cygwin has its own idea of what a process ID is, independent of
-    # the underlying operating system. The Cygwin Perl implements this,
-    # so if we're Cygwin we need to compensate. This MUST return the
-    # Windows-native form under Cygwin, which means any variant which
-    # needs another form must override.
-
-    if ( $^O eq 'cygwin' ) {
-	*My_Pid = sub {
-	    return Cygwin::pid_to_winpid( $$ );
-	};
-    } else {
-	*My_Pid = sub {
-	    return $$;
-	};
-    }
-    %variant_support = (
-	NT => {
-	    check_support => sub {
-		local $@;
-		eval {
-		    require Win32;
-		    Win32->can( 'IsWinNT' ) && Win32::IsWinNT();
-		} or return "$^O is not a member of the Windows NT family";
-		eval { require Win32::API; 1 }
-		    or return 'I can not find Win32::API';
-		my @path = File::Spec->path();
+%variant_support = (
+    NT => {
+	check_support => sub {
+		return "Your OS is not a member of the Windows NT family"
+		    unless Win32->can ('IsWinNT') && Win32::IsWinNT ();
+		return "I can not find Win32::API"
+		    unless require Win32::API;
+		my @path = split ';', $ENV{Path};
 DLL_LOOP:
 		foreach my $dll (qw{PSAPI.DLL ADVAPI32.DLL KERNEL32.DLL}) {
 		    foreach my $loc (@path) {
 			next DLL_LOOP if -e File::Spec->catfile ($loc, $dll);
-		    }
+			}
 		    return "I can not find $dll";
-		}
+		    }
 		return 0;
-	    },
-	    make => sub {
+		},
+	make => sub {
 		require Win32::Process::Info::NT;
 		Win32::Process::Info::NT->new (@_);
-	    },
-	    unsupported => "Disallowed on load of @{[__PACKAGE__]}.",
+		},
+	unsupported => "Disallowed on load of @{[__PACKAGE__]}.",
 	},
-	PT => {
-	    check_support => sub {
-		local $@;
-		return "Unable to load Proc::ProcessTable"
-		    unless eval {require Proc::ProcessTable; 1};
+    WMI => {
+	check_support => sub {
+		return "Unable to load Win32::OLE"
+		    unless eval {require Win32::OLE};
 		return 0;
-	    },
-	    make => sub {
-		require Win32::Process::Info::PT;
-		Win32::Process::Info::PT->new (@_);
-	    },
-	    unsupported => "Disallowed on load of @{[__PACKAGE__]}.",
-	},
-	WMI => {
-	    check_support => sub {
-		local $@;
-		_isReactOS()
-		    and return 'Unsupported under ReactOS';
-		eval {
-		    require Win32::OLE;
-		    1;
-		} or return 'Unable to load Win32::OLE';
-		my ( $wmi, $proc );
-		my $old_warn = Win32::OLE->Option( 'Warn' );
-		eval {
-		    Win32::OLE->Option( Warn => 0 );
-		    $wmi = Win32::OLE->GetObject(
-			'winmgmts:{impersonationLevel=impersonate,(Debug)}!//./root/cimv2'
-		    );
-		    $wmi and $proc = $wmi->Get(
-			sprintf q{Win32_Process='%s'}, __PACKAGE__->My_Pid()
-		    );
-		};
-		Win32::OLE->Option( Warn => $old_warn );
-		$wmi or return 'Unable to get WMI object';
-		$proc or return 'WMI broken: unable to get process object';
-		return 0;
-	    },
-	    make => sub {
+		},
+	make => sub {
 		require Win32::Process::Info::WMI;
 		Win32::Process::Info::WMI->new (@_);
-	    },
-	    unsupported => "Disallowed on load of @{[__PACKAGE__]}.",
+		},
+	unsupported => "Disallowed on load of @{[__PACKAGE__]}.",
 	},
     );
 }
+sub _check_variant {
+croak "Error - Variant '$_[0]' is unknown."
+    unless exists $variant_support{$_[0]};
+exists $variant_support{$_[0]}{unsupported} or croak <<eod;
+Error - Variant '$_[0]' support status is unknown. This can happen if
+        you 'use Win32::Process::Info ();'. Please do not do that.
+eod
+croak "Error - Variant '$_[0]' is unsupported on your configuration. $variant_support{$_[0]}{unsupported}"
+    if $variant_support{$_[0]}{unsupported};
+return 1;
+}
 
-our %mutator = (
+%mutator = (
     elapsed_in_seconds	=> sub {$_[2]},
     variant		=> sub {
-	ref $_[0]
-	    and eval { $_[0]->isa( 'Win32::Process::Info' ) }
-	    or croak 'Variant can not be set on an instance';
+	croak "Error - Variant can not be set on an instance."
+	    if isa ($_[0], 'Win32::Process::Info');
 	foreach (split '\W+', $_[2]) {
-	    my $status;
-	    $status = variant_support_status( $_ )
-		and croak "Variant '$_' unsupported on your configuration; ",
-		    $status;
-	}
-	$_[2]
-    },
-);
+	    _check_variant ($_);
+	    }
+	$_[2]},
+    );
 
 
 =item $pi = Win32::Process::Info->new ([machine], [variant], [hash])
@@ -222,22 +286,14 @@ Windows 2000. This variant does not support connecting to
 another machine, so the 'machine' argument must be an
 empty string (or undef, if you prefer).
 
-PT - Uses Dan Urist's Proc::ProcessTable, making it possible
-(paradoxically) to use this module on other operating systems than
-Windows. Only those Proc::ProcessTable::Process fields which seem
-to correspond to WMI items are returned. B<Caveat:> the PT variant
-is to be considered experimental, and may be changed or retracted
-in future releases.
-
 WMI - Uses the Windows Management Implementation. Good on Win2K, ME,
 and possibly others, depending on their vintage and whether
 WMI has been retrofitted.
 
 The initial default variant comes from environment variable
 PERL_WIN32_PROCESS_INFO_VARIANT. If this is not found, it will be
-'WMI,NT,PT', which means to try WMI first, NT if WMI fails, and PT as a
-last resort. This can be changed using Win32::Process::Info->Set
-(variant => whatever).
+'WMI,NT', which means to try WMI first, and NT if WMI fails. This can
+be changed using Win32::Process::Info->Set (variant => whatever).
 
 The hash argument is a hash reference to additional arguments, if
 any. The hash reference can actually appear anywhere in the argument
@@ -268,44 +324,43 @@ variant in use does not support.
 
 my @argnam = qw{host variant};
 sub new {
-    my ($class, @params) = @_;
-    $class = ref $class if ref $class;
-    my %arg;
-    my ( $self, @probs );
+my $class = shift;
+$class = ref $class if ref $class;
+my %arg;
+my ($self, @probs, $variant);
 
-    my $inx = 0;
-    foreach my $inp (@params) {
-	if (ref $inp eq 'HASH') {
-	    foreach my $key (keys %$inp) {$arg{$key} = $inp->{$key}}
-	} elsif (ref $inp) {
-	    croak "Argument may not be @{[ref $inp]} reference.";
-	} elsif ($argnam[$inx]) {
-	    $arg{$argnam[$inx]} = $inp;
-	} else {
-	    croak "Too many positional arguments.";
+my $inx = 0;
+foreach my $inp (@_) {
+    if (ref $inp eq 'HASH') {
+	foreach my $key (keys %$inp) {$arg{$key} = $inp->{$key}}
 	}
-	$inx++;
+      elsif (ref $inp) {
+	croak "Error - Argument may not be ${[ref $inp]} reference.";
+	}
+      elsif ($argnam[$inx]) {
+	$arg{$argnam[$inx]} = $inp;
+	}
+      else {
+	croak "Error - Too many positional arguments.";
+	}
+    $inx++;
     }
 
-    _import_done()
-	or croak __PACKAGE__,
-	    '->import() must be called before calling ', __PACKAGE__,
-	    '->new()';
-    my $mach = $arg{host} or delete $arg{host};
-    my $try = $arg{variant} || $static{variant} || 'WMI,NT,PT';
-    foreach my $variant (grep {$_} split '\W+', $try) {
-	my $status;
-	$status = variant_support_status( $variant ) and do {
-	    push @probs, $status;
-	    next;
-	};
-	my $self;
-	$self = $variant_support{$variant}{make}->( \%arg ) and do {
-	    $static{variant} ||= $self->{variant} = $variant;
+my $mach = $arg{host} or delete $arg{host};
+my $try = $arg{variant} || $static{variant} || 'WMI,NT';
+foreach $variant (grep {$_} split '\W+', $try) {
+    eval {
+	_check_variant ($variant);
+	$self = $variant_support{$variant}{make}->(\%arg);
 	};
+    if ($self) {
+	$static{variant} ||= $variant;
+	$self->{variant} = $variant;
 	return $self;
+	}
+    push @probs, $@;
     }
-    croak join '; ', @probs;
+croak @probs;
 }
 
 =item @values = $pi->Get (attributes ...)
@@ -336,10 +391,10 @@ not available as a class attribute.
 =cut
 
 sub Get {
-my ($self, @args) = @_;
+my $self = shift;
 $self = \%static unless ref $self;
 my @vals;
-foreach my $name (@args) {
+foreach my $name (@_) {
     croak "Error - Attribute '$name' does not exist."
 	unless exists $self->{$name};
     croak "Error - Attribute '$name' is private."
@@ -380,15 +435,15 @@ nothing useful back.
 =cut
 
 sub Set {
-my ($self, @args) = @_;
+my $self = shift;
 croak "Error - Set requires an even number of arguments."
-    if @args % 2;
+    if @_ % 2;
 $self = \%static unless ref $self;
 my $mutr = $self->{_mutator} || \%mutator;
 my @vals;
-while (@args) {
-    my $name = shift @args;
-    my $val = shift @args;
+while (@_) {
+    my $name = shift;
+    my $val = shift;
     croak "Error - Attribute '$name' does not exist."
 	unless exists $self->{$name};
     croak "Error - Attribute '$name' is read-only."
@@ -460,22 +515,21 @@ sub GetProcInfo {
 =item Win32::Process::Info->import ()
 
 The purpose of this static method is to specify which variants of the
-functionality are legal to use. Possible arguments are 'NT', 'WMI',
-'PT', or some combination of these (e.g. ('NT', 'WMI')). Unrecognized
-arguments are ignored, though this may change if this class becomes a
-subclass of Exporter. If called with no arguments, it is as though it
-were called with arguments ('NT', 'WMI', 'PT'). See L</BUGS>, below, for
-why this mess was introduced in the first place.
+functionality are legal to use. Possible arguments are 'NT', 'WMI', or
+both (i.e. ('NT', 'WMI')). Unrecognized arguments are ignored, though
+this may change if this class becomes a subclass of Exporter. If called
+with no arguments, it is as though it were called with arguments ('NT',
+'WMI'). See L</BUGS>, below, for why this mess was introduced in the
+first place.
 
 This method must be called at least once, B<in a BEGIN block>, or B<no>
 variants will be legal to use. Usually it does B<not> need to be
 explicitly called by the user, since it is called implicitly when you
-C<use Win32::Process::Info;>. If you C<require Win32::Process::Info;>
-you B<will> have to call this method explicitly.
+'use Win32::Process::Info;'.
 
 If this method is called more than once, the second and subsequent calls
 will have no effect on what variants are available. The reason for this
-will be made clear (I hope!) under L</USE IN OTHER MODULES>, below.
+is will be made clear (I hope!) under L</USE IN OTHER MODULES>, below.
 
 The only time a user of this module needs to do anything different
 versus version 1.006 and previous of this module is if this module is
@@ -495,23 +549,59 @@ require rather than use, follow the require with
 
 passing any necessary arguments.
 
+The only time a normal user will be
+
+The purpose of this method is to be specify which
+
+This static method should B<not> normally be called by the user. Its
+purpose is to implement variant restriction on 'use
+Win32::Process::Info'. This works like so:
+
+ use Win32::Process::Info;
+
+allows any variant to be used. It is equivalent to
+
+ use Win32::Process::Info qw{WMI NT};
+
+On the other hand,
+
+ use Win32::Process::Info qw{NT};
+
+allows only the NT variant to be used, which currently is a requirement
+if you wish to fork() (see L</BUGS>).
+
+ use Win32::Process::Info ();
+
+leaves any  and all variants disallowed, because this method is not
+called. You would want to do something like this if you were
+subclassing. See L</NOTICE> above for the details.
+
+This method only has effect the first time it is called. Subsequent
+calls are silently ignored. So
+
+ use Win32::Process::Info qw{NT};
+ use Win32::Process::Info wq{WMI};
+
+leaves only the NT variant allowed.
+
 =cut
 
 {	# Begin local symbol block.
 
     my $idempotent;
 
-    sub import {	## no critic (RequireArgUnpacking)
-	my ($pkg, @params) = @_;
+    sub import {
+	my $pkg = shift;
 	my (@args, @vars);
-	foreach (@params) {
+	foreach (@_) {
 	    if (exists $variant_support{$_}) {
 		push @vars, $_;
 	    } else {
 		push @args, $_;
 	    }
 	}
-
+	# Note that if we ever become a subclass of Exporter
+	# we will have to call __PACKAGE__->SUPER::import (@args);
 	if ($idempotent++) {
 	    # Warning here maybe?
 	} else {
@@ -522,31 +612,10 @@ passing any necessary arguments.
 		    $variant_support{$try}{check_support}->()} || $@;
 	    }
 	}
-
-	return;
-
-#	Do this if we become a subclass of Exporter
-#	@_ = ( $pkg, @args );
-#	goto &Exporter::import;;
-    }
-
-    # Return the number of times import() done.
-    sub _import_done {
-	return $idempotent;
     }
 
 }	# End local symbol block.
 
-
-{
-    my $is_reactos = $^O eq 'MSWin32' &&
-	defined $ENV{OS} && lc $ENV{OS} eq 'reactos';
-    sub _isReactOS {
-	return $is_reactos;
-    }
-}
-
-
 =item %subs = $pi->Subprocesses ([pid ...])
 
 This method takes as its argument a list of PIDs, and returns a hash
@@ -566,13 +635,12 @@ support this. If the variant you're using doesn't support this
 attribute, you get back an empty hash. Specifically:
 
  NT -> unsupported
- PT -> supported
  WMI -> supported
 
 =cut
 
 sub Subprocesses {
-my ($self, @args) = @_;
+my $self = shift;
 my %prox = map {($_->{ProcessId} => $_)}
 	@{$self->GetProcInfo ({no_user_info => 1})};
 my %subs;
@@ -580,120 +648,26 @@ my $rslt = \%subs;
 my $key_found;
 foreach my $proc (values %prox) {
     $subs{$proc->{ProcessId}} ||= [];
-    # TRW 1.011_01 next unless $proc->{ParentProcessId};
-    defined (my $pop = $proc->{ParentProcessId}) or next; # TRW 1.011_01
+    next unless $proc->{ParentProcessId};
     $key_found++;
-    # TRW 1.011_01 next unless $prox{$proc->{ParentProcessId}};
-    $prox{$pop} or next;	# TRW 1.011_01
-# TRW 1.012_02    $proc->{CreationDate} >= $prox{$pop}{CreationDate} or next;	# TRW 1.011_01
-    (defined($proc->{CreationDate}) &&
-	defined($prox{$pop}{CreationDate}) && 
-        $proc->{CreationDate} >= $prox{$pop}{CreationDate})
-	or next;	# TRW 1.012_02
-    # TRW 1.011_01 push @{$subs{$proc->{ParentProcessId}}}, $proc->{ProcessId};
-    push @{$subs{$pop}}, $proc->{ProcessId};
+    next unless $prox{$proc->{ParentProcessId}};
+    push @{$subs{$proc->{ParentProcessId}}}, $proc->{ProcessId};
     }
 my %listed;
 return %listed unless $key_found;
-if (@args) {
+if (@_) {
     $rslt = \%listed;
-    while (@args) {
-	my $pid = shift @args;
-	next unless $subs{$pid};	# TRW 1.006
+    while (@_) {
+	my $pid = shift;
+    next unless $subs{$pid};	# TRW 1.006
 	next if $listed{$pid};
 	$listed{$pid} = $subs{$pid};
-	push @args, @{$subs{$pid}};
+	push @_, @{$subs{$pid}};
 	}
     }
 return wantarray ? %$rslt : $rslt;
 }
 
-=item @info = $pi->SubProcInfo ();
-
-This is a convenience method which wraps GetProcInfo(). It has the same
-calling sequence, and returns generally the same data. But the data
-returned by this method will also have the {subProcesses} key, which
-will contain a reference to an array of hash references representing the
-data on subprocesses of each process.
-
-Unlike the data returned from Subprocesses(), the data here are not
-flattened; so if you specify one or more process IDs as arguments, you
-will get back at most the number of process IDs you specified; fewer if
-some of the specified processes do not exist.
-
-B<Note well> that a given process can occur more than once in the
-output. If you call SubProcInfo without arguments, the @info array will
-contain every process in the system, even those which are also in some
-other process' {subProcesses} array.
-
-Also unlike Subprocesses(), you will get an exception if you use this
-method with a variant that does not support the ParentProcessId key.
-
-=cut
-
-sub SubProcInfo {
-    my ($self, @args) = @_;
-    my $opt = ref $args[0] eq 'HASH' ? shift @args : {};
-    my @data = $self->GetProcInfo ($opt);
-    my %subs = map {$_->{ProcessId} => $_} @data;
-    my $bingo;
-    foreach my $proc (@data) {
-	exists $proc->{ParentProcessId} or next;
-	$proc->{subProcesses} ||= [];
-	$bingo++;
-	defined (my $dad = $subs{$proc->{ParentProcessId}}) or next;
-	(defined $dad->{CreationDate} && defined $proc->{CreationDate})
-	    or next;
-	$dad->{CreationDate} > $proc->{CreationDate} and next;
-	push @{$dad->{subProcesses} ||= []}, $proc;
-    }
-    $bingo or croak "Error - Variant '@{[$self->Get('variant')
-    ]}' does not support the ParentProcessId key";
-    if (@args) {
-	return (map {exists $subs{$_} ? $subs{$_} : ()} @args);
-    } else {
-	return @data;
-    }
-}
-
-=item $pid = $pi->My_Pid()
-
-This convenience method returns the process ID of the current process,
-in a form appropriate to the operating system and the variant in use.
-Normally, it simply returns C<$$>. But Cygwin has its own idea of what
-the process ID is, which may differ from Windows. Worse than that, under
-Cygwin the NT and WMI variants return Windows PIDs, while PT appears to
-return Cygwin PIDs.
-
-=cut
-
-# This is defined above, trickily, as an assignment to *My_Pid, so we
-# don't have to test $^O every time. It's above because code in a BEGIN
-# block needs it.
-
-=item $text = Win32::Process::Info->variant_support_status($variant);
-
-This static method returns the support status of the given variant. The
-return is false if the variant is supported, or an appropriate message
-if the variant is unsupported.
-
-This method can also be called as a normal method, or even as a
-subroutine.
-
-=cut
-
-sub variant_support_status {
-    my @args = @_;
-    my $variant = pop @args or croak "Variant not specified";
-    exists $variant_support{$variant}
-	or croak "Variant '$variant' is unknown";
-    _import_done()
-	or croak __PACKAGE__,
-	    '->import() must be called before calling ', __PACKAGE__,
-	    '->variant_support_status()';
-    return $variant_support{$variant}{unsupported};
-}
-
 =item print "$pi Version = @{[$pi->Version ()]}\n"
 
 This method just returns the version number of the
@@ -705,6 +679,19 @@ sub Version {
 return $Win32::Process::Info::VERSION;
 }
 
+#	$result = $self->_available (variant)
+#	is a debugging tool that returns the content of {unsupported}
+#	for the given variant. If the variant does not exist, you get
+#	undef.
+
+sub _available {
+    my $pkg = shift;
+    my $var = shift;
+    $variant_support{$var} ?
+	$variant_support{$var}{unsupported} :
+	undef; 
+}
+
 #
 #	$self->_build_hash ([hashref], key, value ...)
 #	builds a process info hash out of the given keys and values.
@@ -715,11 +702,11 @@ return $Win32::Process::Info::VERSION;
 #	Either way, the hash is returned.
 
 sub _build_hash {
-my ($self, $hash, @args) = @_;
-$hash ||= {};
-while (@args) {
-    my $key = shift @args;
-    my $val = shift @args;
+my $self = shift;
+my $hash = shift || {};
+while (@_) {
+    my $key = shift;
+    my $val = shift;
     $val = $self->{_xfrm}{$key}->($self, $val)
 	if (exists $self->{_xfrm}{$key});
     $hash->{$key} = $val;
@@ -746,13 +733,13 @@ return wantarray ? @_ : $_[0];
 
 
 sub _date_to_time_t {
-my ($self, @args) = @_;
+my $self = shift;
 my @result;
-local $^W = 0;	# Prevent Time::Local 1.1 from complaining. This appears
-		# to be fixed in 1.11, but since Time::Local is part of
+local $^W;	# Prevent Time::Local 1.1 from complaining. This appears
+$^W = 0;	# to be fixed in 1.11, but since Time::Local is part of
 		# the ActivePerl core, there's no PPM installer for it.
 		# At least, not that I can find.
-foreach (@args) {
+foreach (@_) {
     if ($_) {
 	my ($yr, $mo, $da, $hr, $mi, $sc) = m/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/;
 	--$mo;
@@ -783,19 +770,11 @@ implemented as subclasses of this module).
 Modules that simply make use of this module (the 'has-a' relationship)
 should work as before, B<provided> they 'use Win32::Process::Info'. Note
 that the phrase 'as before' is literal, and means (among other things),
-that you can't use the emulated fork.
+that you can't use the emulated fork. Modules that 'require
+Win32::Process::Info' should call its import() method explicitly, as
+should modules that want to allow the emulated fork.
 
-If you as the author of a module that uses Win32::Process::Info wish to
-allow emulated forks, you have a number of options.
-
-The easiest way to go is
-
- use Win32::Process::Info qw{NT};
-
-if this provides enough information for your module.
-
-If you would prefer the extra information provided by WMI but can
-operate in a degraded mode if needed, you can do something like
+The easiest way to do this is probably something like
 
  use Win32::Process::Info ();
  
@@ -822,7 +801,6 @@ with fork, you will of course need to
 
  use Win32::Process::Info qw{NT};
 
-This is why only the first import() sets the possible variants.
 
 =head1 ENVIRONMENT
 
@@ -863,14 +841,69 @@ This library uses the following libraries:
 
  Carp
  Time::Local
- Proc::ProcessTable (if using the PT variant)
  Win32::API (if using the NT-native variant)
  Win32API::Registry (if using the NT-native variant)
  Win32::ODBC (if using the WMI variant)
 
 As of ActivePerl 630, none of this uses any packages that are not
-included with ActivePerl. Carp and Time::Local have been in the core
-since at least 5.004. Your mileage may, of course, vary.
+included with ActivePerl. Your mileage may vary.
+
+=head1 HISTORY
+
+ 0.010 Released as Win32::Process::Info
+ 0.011 Added Version method
+       Fixed warning in NT.pm when -w in effect. Fix provided
+           by Judy Hawkins (of Pitney Bowes, according to her
+           mailing address), and accepted with thanks.
+ 0.012 Hid attributes beginning with "_".
+ 0.013 Use environment variable PERL_WIN32_PROCESS_INFO_VARIANT
+           to specify the default variant list.
+       Add a hash reference argument to new (); use this to
+           specify username and password to the WMI variant.
+       Turn on debug privilege in NT variant. This also resulted
+           in dependency on Win32API::Registry.
+       Return OwnerSid and Owner in NT variant.
+ 0.014 Remove conditional dependencies from Makefile.PL
+       Track changes in Win32::API. Can no longer "require" it.
+       WMI variant no longer asserts debug privilege by default.
+       Use environment variable PERL_WIN32_PROCESS_INFO_WMI_DEBUG
+          to tell the WMI variant whether to assert debug.
+       Use environment variable PERL_WIN32_PROCESS_INFO_WMI_PARIAH
+          to encode processes to skip when determining the owner.
+       Add optional first hash ref argument to GetProcInfo.
+       Add Subprocesses method.
+ 1.000 Add assert_debug_priv hash argument to the 'new' method.
+       Fix documentation, both pod errors and actual doc bugs.
+       When the only thing you've done in two months is add a
+           semicolon to a comment, it's probably time to call it
+           production code.
+ 1.001 Removed dependency on Win32. We still need it, of course,
+       but PPM3 chokes on it, and I figure anyone who IS using
+       PPM3 already has it, and anyone who ISN'T is smart enough
+       to figure out what's going on - or at least read the Readme.
+ 1.002 Document leaky behavior of WMI variant, and try to make it
+       leak less. Document related modules.
+ 1.003 Documented PERL_WIN32_PROCESS_INFO_WMI_DEBUG correctly
+       (the docs had a _PRIV on the end).
+       Recorded the variant name in the object.
+       Clarified (hopefully) the docs on how to use the
+       PERL_WIN32_PROCESS_INFO_VARIANT environment variable.
+       Added the current status and whereabouts of
+       Win32::IProc. Thanks to Eric Bluestein
+       (http://www.emblue.com/) for pointing this out.
+ 1.004 Remove commented-out dependencies in Makefile.PL,
+       since CPAN's now checking. The only one that really
+       counts is Win32, but ActiveState's PPM3 chokes on
+       this, or at least did as of January 2001.
+ 1.005 Addressed token handle leak in the NT variant.
+ 1.006 Silently skip non-existent processes in the Subprocesses
+       method. Fix provided by Kirk Baucom of Itron, and accepted
+       with thanks.
+ 1.007 Provide method to disallow a given variant when the package is
+       loaded. This prevents the normal testing, which causes problems
+       if fork() is used. Thanks to Malcolm Nooning for finding the
+       problem, helping me work out the solution, and sharing the
+       results of his correspondence with ActiveState.
 
 =head1 BUGS
 
@@ -889,26 +922,17 @@ load this module as
 The problem is that fork() and Win32::OLE (used by the WMI variant) do
 not play B<at all> nicely together. This appears to be an acknowledged
 problem with Win32::OLE, which is brought on simply by loading the
-module. See import() above for the gory details.
-
-The use of the NT and WMI variants under non-Microsoft systems is
-unsupported. ReactOS 0.3.3 is known to lock up when GetProcInfo() is
-called; since this  works on the Microsoft OSes, I am inclined to
-attribute this to the acknowledged alpha-ness of ReactOS. I have no idea
-what happens under Wine. B<Caveat user.>
+module. See import() above for the consequences of handling things this
+way.
 
 Bugs can be reported to the author by mail, or through
 L<http://rt.cpan.org>.
 
 =head1 RESTRICTIONS
 
-You can not C<require> this module except in a BEGIN block. This is a
-consequence of the use of Win32::API, which has the same restriction, at
-least in some versions.
-
-If you C<require> this module, you B<must> explicitly call C<<
-Win32::Process::Info->import() >>, so that the module will know what
-variants are available.
+You can not "require" this library except in a BEGIN block. This is a
+consequence of the use of Win32::API, which has the same restriction,
+at least in some versions.
 
 If your code calls fork (), you must load this module as
 
@@ -924,12 +948,6 @@ ones will.
 
 =over 4
 
-=item Proc::ProcessTable by Dan Urist
-
-This module does not as of this writing support Windows, though there
-is a minimal Cygwin version that might serve as a starting point. The
-'PT' variant makes use of this module.
-
 =item Win32::PerfLib by Jutta M. Klebe
 
 This module focuses on performance counters. It is a ".xs" module,
@@ -985,21 +1003,16 @@ NT 4.0 without WMI.
 
 Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT
 
-Copyright (C) 2001-2005 by E. I. DuPont de Nemours and Company, Inc. All
-rights reserved.
+Copyright 2001, 2002, 2003, 2004, 2005 by E. I. DuPont de Nemours and
+Company, Inc.  All rights reserved.
 
-Copyright (C) 2007-2011, 2013-2014 by Thomas R. Wyant, III
+Modifications since version 1.006 copyright 2007 by Thomas R. Wyant,
+III. All rights reserved.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl 5.10.0. For more details, see the full text
-of the licenses in the directory LICENSES.
-
-This program is distributed in the hope that it will be useful, but
-without any warranty; without even the implied warranty of
-merchantability or fitness for a particular purpose.
+This module is free software; you can use it, redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =cut
 
-# ex: set textwidth=72 :
