@@ -1,3 +1,6 @@
+use strict;
+use warnings;
+
 =head1 NAME
 
 Win32::Process::Info::NT - Provide process information via NT-native calls.
@@ -10,7 +13,7 @@ machine, using Microsoft Windows NT's native process
 information calls.
 
  use Win32::Process::Info
- $pi = Win32::Process::Info->new (undef, 'NT');
+ $pi = Win32::Process::Info->new ([machine], 'NT');
  $pi->Set (elapsed_as_seconds => 0);	# In clunks, not seconds.
  @pids = $pi->ListPids ();	# Get all known PIDs
  @info = $pi->GetProcInfo ();	# Get the max
@@ -28,13 +31,11 @@ only be called via that package.
 
 =head1 DESCRIPTION
 
-The main purpose of the Win32::Process::Info::NT package is to get whatever
+The main purpose of the Win32::Process::Info package is to get whatever
 information is convenient (for the author!) about one or more Windows
 32 processes. GetProcInfo (which see) is therefore the most-important
 subroutine in the package. See it for more information.
 
-This package returns Windows process IDs, even under Cygwin.
-
 Unless explicitly stated otherwise, modules, variables, and so
 on are considered private. That is, the author reserves the right
 to make arbitrary changes in the way they work, without telling
@@ -51,19 +52,13 @@ The following subroutines should be considered public:
 
 =cut
 
-
-package Win32::Process::Info::NT;
-
-use 5.006;
-
-use strict;
-use warnings;
+package Win32::Process::Info::DummyRoutine;
 
 #	The purpose of this is to provide a dummy Call
 #	method for those cases where we might not be able
 #	to map a subroutine.
 
-sub Win32::Process::Info::DummyRoutine::new {
+sub new {
 my $class = shift;
 $class = ref $class if ref $class;
 my $self = {};
@@ -71,47 +66,116 @@ bless $self, $class;
 return $self;
 }
 
-sub Win32::Process::Info::DummyRoutine::Call {
-return undef;	## no critic (ProhibitExplicitReturnUndef)
+sub Call {
+return undef;
 }
 
-use base qw{ Win32::Process::Info };
+# 0.010	02-Sep-2002	T. R. Wyant
+#		Initial release under this name.
+#
+# 0.011	14-Sep-2002	T. R. Wyant
+#		Increment version.
+#
+#	30-Oct-2002	T. R. Wyant
+#		Fix warning when -w in effect. Fix provided by Judy
+#		Hawkins (of Pitney Bowes, according to her mailing
+#		address), and accepted with thanks.
+#
+# 0.012	11-Nov-2002	T. R. Wyant
+#		Increment version.
+#		Pick up username info.
+#
+#	12-Nov-2002	T. R. Wyant
+#		Change 'new' argument to a hash. This method wasn't
+#			marked private, but neither is it in the POD.
+#		Reject hashes with keys other than variant.
+#
+#	21-Nov-2002	T. R. Wyant
+#		Set SeDebugPrivilege on instantiation.
+#		Replace "sub" manifest constants with "use constant".
+#		Retry OpenProcess in (apparently futile) attempt to
+#			get at lest _some_ information when it's
+#			not all available.
+#		GetProcInfo now returns all PIDs requested, even if
+#			the process handle could not be opened. This
+#			is what the documentation has said it did all
+#			along, but was not what it actually did.
+#
+#	05-Dec-2002	T. R. Wyant
+#		Discovered Win32API::Registry::AllowPriv. Used that
+#			instead of my own Win32::API method to set
+#			the privilege (when you care enough to steal
+#			the very best). My own method preserved after
+#			the __END__ directive as a monument to - well,
+#			something.
+#
+# 0.013	13-Mar-2003	T. R. Wyant
+#		Changed dependencies in Makefile.pl. See Info.pm for
+#			more information.
+#		Made Win32API::Registry optional; if not found, just
+#			don't set SeDebugPrivilege.
+#
+# 0.013_23 26-Jun-2003	T. R. Wyant
+#		Added optional first hashref argument to GetProcInfo,
+#			used no_user_info key to bypass user info. This
+#			was to be consistent with the WMI variant; the
+#			reason for adding this to WMI doesn't exist for
+#			this variant.
+#
+# 0.014	27-Jun-2003	T. R. Wyant
+#		Released.
+#
+# 0.014_01 25-Jul-2003	T. R. Wyant
+#		Added hash argument assert_debug_priv to 'new' method.
+#			It's ignored, though, since this variant
+#			always asserts debug.
+#
+# 1.000 09-Oct-2003	T. R. Wyant
+#		When the only thing you've fixed in the last two months
+#			is the docs, it's time to call it production
+#			code. And if _that_ statement doesn't flush
+#			out more problems, nothing will.
+#
+# 1.005 15-Mar-2005	T. R. Wyant
+#		Move assertion of seDebugPriv to the point where we
+#		discover we can require Win32API::Registry, since we
+#		don't need to do it but once for the process (not once
+#		per instantiation of the object), and we don't leak
+#		token handles this way.
 
-our $VERSION = '1.020_01';
+package Win32::Process::Info::NT;
 
-our $AdjustTokenPrivileges;
-our $CloseHandle;
-our $elapsed_in_seconds;
-our $EnumProcesses;
-our $EnumProcessModules;
-our $FileTimeToSystemTime;
-our $GetCurrentProcess;
-our $GetModuleFileNameEx;
-our $GetPriorityClass;
-our $GetProcessAffinityMask;
-our $GetProcessIoCounters;
-our $GetProcessWorkingSetSize;
-our $GetProcessTimes;
-our $GetProcessVersion;
-our $GetTokenInformation;
-our $LookupAccountSid;
-our $LookupPrivilegeValue;
-our $OpenProcess;
-our $OpenProcessToken;
-
-our $GetSidIdentifierAuthority;
-our $GetSidSubAuthority;
-our $GetSidSubAuthorityCount;
-our $IsValidSid;
+use base qw{Win32::Process::Info};
+use vars qw{$VERSION};
+$VERSION = 1.005;
 
+use vars qw {
+    $AdjustTokenPrivileges
+    $CloseHandle
+    $elapsed_in_seconds
+    $EnumProcesses
+    $EnumProcessModules
+    $FileTimeToSystemTime
+    $GetCurrentProcess
+    $GetModuleFileNameEx
+    $GetPriorityClass
+    $GetProcessAffinityMask
+    $GetProcessIoCounters
+    $GetProcessWorkingSetSize
+    $GetProcessTimes
+    $GetProcessVersion
+    $GetTokenInformation
+    $LookupAccountSid
+    $LookupPrivilegeValue
+    $OpenProcess
+    $OpenProcessToken
+    $VERSION
+    };
 use Carp;
 use File::Basename;
 use Win32;
 use Win32::API;
 
-use constant TokenUser => 1;	# PER MSDN
-use constant TokenOwner => 4;
-
 my $setpriv;
 eval {
     require Win32API::Registry and
@@ -181,8 +245,9 @@ list of process IDs, and get out a list of the attributes of
 all such processes that actually exist. If you call this
 method in scalar context, you get a reference to the list.
 
-What keys are available depend on the variant in use. With the NT
-variant you can hope to get at least the following keys for a "normal"
+What keys are available depend both on the variant in use and the
+setting of b<use_wmi_names>. Assuming B<use_wmi_names> is TRUE,
+you can hope to get at least the following keys for a "normal"
 process (i.e. not the idle process, which is PID 0, nor the system,
 which is _usually_ PID 8) to which you have access:
 
@@ -310,8 +375,8 @@ use constant TOKEN_EXECUTE         => STANDARD_RIGHTS_EXECUTE;
 # Pointer	P
 
 sub GetProcInfo {
-my ( $self, @args ) = @_;
-my $opt = ref $args[0] eq 'HASH' ? shift @args : {};
+my $self = shift;
+my $opt = ref $_[0] eq 'HASH' ? shift : {};
 
 $CloseHandle ||= _map ('KERNEL32', 'CloseHandle', [qw{N}], 'V');
 $GetModuleFileNameEx ||=
@@ -340,21 +405,20 @@ $EnumProcessModules ||=
 my $dac = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
 my $tac = TOKEN_READ;
 
-@args or @args = ListPids ($self);
+@_ = ListPids ($self) unless @_;
 
 my @pinf;
 
 my $dat;
-my $my_pid = $self->My_Pid();
 my %sid_to_name;
 my @trydac = (
     PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
     PROCESS_QUERY_INFORMATION,
     );
 
-foreach my $pid (map {$_ eq '.' ? $my_pid : $_} @args) {
+foreach my $pid (map {$_ eq '.' ? $$ : $_} @_) {
 
-    local $^E = 0;
+    $^E = 0;
     $dat = $self->_build_hash (undef, ProcessId => $pid);
     $self->_build_hash ($dat, Name => 'System Idle Process')
 	unless $pid;
@@ -420,6 +484,8 @@ foreach my $pid (map {$_ eq '.' ? $my_pid : $_} @args) {
 	last if $opt->{no_user_info};
 	$OpenProcessToken->Call ($prchdl, $tac, $tokhdl)
 	    or do {$tokhdl = undef; last; };
+	sub TokenUser {1};	# PER MSDN
+	sub TokenOwner {4};
 	my ($dsize, $size_in, $size_out, $sid, $stat, $use, $void);
 	$tokhdl = unpack 'L', $tokhdl;
 
@@ -469,13 +535,11 @@ return wantarray ? @pinf : \@pinf;
 }
 
 sub _to_char_date {
-my @args = @_;
 my @result;
-( $FileTimeToSystemTime ||=
-    Win32::API->new ('KERNEL32', 'FileTimeToSystemTime', [qw{P P}], 'I') )
-    or croak "Error - Failed to map FileTimeToSystemTime: $^E";
+$FileTimeToSystemTime ||= Win32::API->new ('KERNEL32', 'FileTimeToSystemTime', [qw{P P}], 'I') or
+    croak "Error - Failed to map FileTimeToSystemTime: $^E";
 my $systim = '  ' x 8;
-foreach (@args) {
+foreach (@_) {
     $FileTimeToSystemTime->Call ($_, $systim) or
 	croak "Error - FileTimeToSystemTime failed: $^E";
     my $time;
@@ -494,9 +558,8 @@ return $result[0];
 }
 
 sub _ll_to_bigint {
-my @args = @_;
 my @result;
-foreach (@args) {
+foreach (@_) {
     my @data = unpack 'L*', $_;
     while (@data) {
 	my $low = shift @data;
@@ -509,9 +572,8 @@ return $result[0];
 }
 
 sub _clunks_to_secs {
-my @args = @_;
 my @result;
-foreach (_ll_to_bigint (@args)) {
+foreach (_ll_to_bigint (@_)) {
     push @result, $_ / 10_000_000;
     }
 return @result if wantarray;
@@ -528,12 +590,9 @@ reference to the list is returned.
 =cut
 
 sub ListPids {
-my ( $self, @args ) = @_;
+my $self = shift;
 my $filter = undef;
-my $my_pid = $self->My_Pid();
-@args and $filter = {
-    map { ($_ eq '.' ? $my_pid : $_) => 1 } @args
-};
+$filter = {map {(($_ eq '.' ? $$ : $_), 1)} @_} if @_;
 $EnumProcesses ||= _map ('PSAPI', 'EnumProcesses', [qw{P N P}], 'I');
 my $psiz = 4;
 my $bsiz = 0;
@@ -556,10 +615,16 @@ my $bsiz = 0;
 	}
     return wantarray ? @pids : \@pids;
     }
-confess 'Programming error - should not get here';
+
 }
 
 
+use vars qw{
+    $GetSidIdentifierAuthority
+    $GetSidSubAuthority
+    $GetSidSubAuthorityCount
+    $IsValidSid
+    };
 
 #	_text_sid (pointer to SID)
 
@@ -587,12 +652,6 @@ confess 'Programming error - should not get here';
 #	appropriate size. A further unpack may be necessary to extract
 #	data from the finally-obtained structure. You'll be seeing a
 #	lot of this pack/unpack idiom in the code that follows.
-#
-#	Interestingly enough in February 2013 I found (fairly easily)
-#	ConvertSidToStringSid(), which seems to do what I need, and
-#	seems to have the same vintage as the other calls used above.
-#	But in September of 2002 when I was writing this code I never
-#	found it - certainly the docs cited never mentioned it.
 
 sub _text_sid {
 my $sid = shift;
@@ -601,8 +660,8 @@ my $sid = shift;
 #	Make sure we have a valid SID
 
 $IsValidSid ||= _map ('ADVAPI32', 'IsValidSid', [qw{P}], 'I');
-my $stat = $IsValidSid->Call ($sid)
-    or return;
+my $stat = $IsValidSid->Call ($sid);
+return undef unless $stat;
 
 
 #	Get the identifier authority.
@@ -611,11 +670,7 @@ $GetSidIdentifierAuthority ||=
 	_map ('ADVAPI32', 'GetSidIdentifierAuthority', [qw{P}], 'N');
 my $sia = $GetSidIdentifierAuthority->Call ($sid);
 $sia = pack 'L', $sia;
-# Occasionally we end up with an undef value here, which indicates a
-# failure. The docs say this only happens with an invalid SID, but what
-# do they know?
-defined( $sia = unpack 'P6', $sia )
-    or return;
+$sia = unpack 'P6', $sia;
 
 
 #	Get the number of subauthorities.
@@ -706,19 +761,12 @@ NT 4.0 without WMI.
 
 Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT
 
-Copyright (C) 2001-2003 by E. I. DuPont de Nemours and Company, Inc.
+Copyright 2001, 2002, 2003 by E. I. DuPont de Nemours and Company, Inc.
 
-Copyright (C) 2007-2011, 2013-2014 by Thomas R. Wyant, III
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl 5.10.0. For more details, see the full text
-of the licenses in the directory LICENSES.
-
-This program is distributed in the hope that it will be useful, but
-without any warranty; without even the implied warranty of
-merchantability or fitness for a particular purpose.
+This module is free software; you can use it, redistribute it
+and/or modify it under the same terms as Perl itself.
 
 =cut
 
