@@ -8,7 +8,7 @@ This package fetches process information on a given Windows
 machine, using Microsoft's Windows Management Implementation.
 
  use Win32::Process::Info
- $pi = Win32::Process::Info->new (undef, 'WMI');
+ $pi = Win32::Process::Info->new ([machine], 'WMI');
  $pi->Set (elapsed_as_seconds => 0);	# In clunks, not seconds.
  @pids = $pi->ListPids ();	# Get all known PIDs
  @info = $pi->GetProcInfo ();	# Get the max
@@ -24,23 +24,137 @@ only be called via that package.
 This package implements the WMI-specific methods of
 Win32::Process::Info.
 
-This package returns Windows process IDs, even under Cygwin.
-
 The following methods should be considered public:
 
 =over 4
 
 =cut
 
+#	Modifications:
+
+# 0.010	02-Sep-2002	T. R. Wyant
+#		Initial release under this name.
+#
+# 0.011	14-Sep-2002	T. R. Wyant
+#		Increment version.
+#
+#	26-Sep-2002	T. R. Wyant
+#		Pull variant construction out of loop; we only need
+#			three of them, anyway.
+#
+#	30-Oct-2002	T. R. Wyant
+#		Set the Warn level to 0 inside GetProcInfo, to try
+#			to supress OLE's annoying exceptions when
+#			getting the SID of a process with no owner
+#			(e.g. 'idle', 'system').
+#
+#	31-Oct-2002	T. R. Wyant
+#		Reinstate check for executable path before getting
+#			user SID and name, as way to bypass system
+#			and idle pseudo-processes.
+#
+#	01-Nov-2002	T. R. Wyant
+#		Set the Warn level to 0 around the instantiation of
+#			the WMI object, to supress yet another annoying
+#			message when WMI not present.
+#
+# 0.012	06-Nov-2002	T. R. Wyant
+#		Increment version number.
+#		Add attributes 'user' and 'password'.
+#		Connect using SWbemLocator.ConnectServer rather than
+#			new-ing the monicker, so we can specify user
+#			and password if we want.
+#
+#	12-Nov-2002	T. R. Wyant
+#		Change 'new' argument to a hash (it _was_ clearly
+#			marked as not being public!)
+#		Reject hashes with keys other than host, variant,
+#			user, and password.
+#
+#	03-Dec-2002	T. R. Wyant
+#		Add ad-hoc check in 'new' to see if we can retrieve
+#			our own proces object. If we can't, we presume
+#			that we can't get at any of the process objects
+#			either, so we croak.
+#		Tried going back to using monicker to instantiate WMI
+#			class if we don't need to specify username and
+#			password. This didn't behave well on the only
+#			NT 4.0 system I have access to. In fact, it
+#			hangs trying to call GetOwnerSid on the time
+#			synchronization service. Using ConnectServer
+#			finds no objects at all - but at least this is
+#			something deterministic that I can check for
+#			and fail over to the NT-native interface.
+#
+#	07-Dec-2002	T. R. Wyant
+#		Got rid of ad-hoc retrieval check in 'new.' It wasn't
+#			doing what I wanted anyway (which was to cope
+#			with a flakey NT 4.0 WMI installation).
+#
+# 0.013	13-Mar-2003	T. R. Wyant
+#		Changed dependencies in Makefile.pl. See Info.pm for
+#			more information. Released to CPAN.
+#
+# 0.013_1 13-Apr-2003	T. R. Wyant
+#		Don't get sid and owner for knlwrap.exe. I have no
+#		logic for this, but I note that pulist doesn't try,
+#		and WBEMTEST fails to record an executable path, which
+#		is my normal ad-hocery for skipping the user fetch.
+#
+# 0.013_2 21-Apr-2003	T. R. Wyant
+#		Build the %pariah hash (processes not to call methods
+#		on) from $ENV{PERL_WIN32_PROCESS_INFO_WMI_PARIAH} if
+#		this is present; otherwise use a hardwired list.
+#
+# 0.013_21 05-Jun-2003	T. R. Wyant
+#		Added nodemgr.exe to %pariah.
+#
+# 0.013_22 20-Jun-2003	T. R. Wyant
+#		Added LUCOMS~1.EXE to %pariah.
+#		Special-case PERL_WIN32_PROCESS_INFO_WMI_PARIAH=*
+#			to mean all processes.
+#		DO NOT assert wbemPrivilegeDebug by default. Do so only
+#			if PERL_WIN32_PROCESS_INFO_WMI_DEBUG is TRUE.
+#		After some testing on the above, empty the %pariah hash.
+#			Note that the previous value was
+#			'dllhost.exe;knlwrap.exe;lucoms~1.exe;nodemgr.exe'.
+#
+# 0.013_23 26-Jun-2003	T. R. Wyant
+#		Added optional first hashref argument to GetProcInfo,
+#			used no_user_info key to bypass user info. This
+#			was in support of the Subprocesses method of
+#			Win32::Process::Info.
+#
+# 0.014	27-Jun-2003	T. R. Wyant
+#		Released.
+#
+# 0.014_01 25-Jul-2003	T. R. Wyant
+#		Added hash argument assert_debug_priv to 'new' method.
+#			This overrides the environment variable.
+#
+# 1.000 09-Oct-2003	T. R. Wyant
+#		When the only thing you've fixed in the last two months
+#			is the docs, it's time to call it production
+#			code. And if _that_ statement doesn't flush
+#			out more problems, nothing will.
+#
+# 1.001_01 24-Feb-2004	T. R. Wyant
+#		Failed attempt to fix WMI memory leak. But it's clearly
+#		associated with the use of Win32::OLE::Variant objects
+#		to retrieve the owner information, because the code
+#		appears not to leak if we don't do this. Versions prior
+#		to this one leaked anyway, because the Variant objects
+#		were allocated whether or not they were used. But this
+#		version doesn't allocate them unless they are to be
+#		used.
+
 package Win32::Process::Info::WMI;
 
+use base qw{Win32::Process::Info};
+use vars qw{$VERSION};
+$VERSION = 1.001_01;
+
 use strict;
-use warnings;
-
-use base qw{ Win32::Process::Info };
-
-our $VERSION = '1.020_01';
-
 use vars qw{%mutator};
 use Carp;
 use Time::Local;
@@ -211,10 +325,9 @@ be present.
 
 sub _get_proc_objects {
 my $self = shift;
-my $my_pid = $self->My_Pid();
 my @procs = @_ ?
     map {
-	my $pi = $_ eq '.' ? $my_pid : $_;
+	my $pi = $_ eq '.' ? $$ : $_;
 	my $obj = $self->{wmi}->Get ("Win32_Process='$pi'");
 	Win32::OLE->LastError ? () : ($obj)	
 	} @_ :
@@ -411,20 +524,13 @@ published by Macmillan Technical Publishing, ISBN 1-57870-215-1
 
 Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT
 
-Copyright (C) 2001-2005 by E. I. DuPont de Nemours and Company, Inc.
+Copyright 2001, 2002, 2003, 2004 by
+E. I. DuPont de Nemours and Company, Inc.
 
-Copyright (C) 2007, 2010-2011, 2013-2014 by Thomas R. Wyant, III
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl 5.10.0. For more details, see the full text
-of the licenses in the directory LICENSES.
-
-This program is distributed in the hope that it will be useful, but
-without any warranty; without even the implied warranty of
-merchantability or fitness for a particular purpose.
+This module is free software; you can use it, redistribute it
+and/or modify it under the same terms as Perl itself.
 
 =cut
 
-# ex: set textwidth=72 :
